#include "chatoperator.h"

#include <QThread>
#include <QTime>
#include <QJsonArray>
#include <QJsonValue>
#include <QJsonObject>
#include <QJsonDocument>

#include <ukui-log4qt.h>

#include "global/kaglobal.h"

bool is_reply_end = true;

class ChatOperator::Private
{
public:
    void* session = nullptr;
};

/**
 * @brief chatCallback
 *          聊天结果返回回调函数，数据流式返回
 * @param text_data
 * @param usr_data
 */
static void chatCallback(NlpTextData text_data,void *usr_data)
{
    ChatOperator *chatOperator = (ChatOperator*)usr_data;
    int errorCode = text_data.error_code;
    char * result = (char *)text_data.text;
    QJsonParseError jsonError;
    QJsonDocument doc = QJsonDocument::fromJson(result, &jsonError);
    if (jsonError.error == QJsonParseError::NoError)
    {
        QJsonObject obj = doc.object();
        chatOperator->is_end = obj.value("is_end").toBool();
        chatOperator->result_msg = obj.value("result").toString();
        chatOperator->result_index = obj.value("sentence_id").toInt();
    }else{
        KyInfo() << "JSON parsing error: " << jsonError.errorString();
    }
    is_reply_end = chatOperator->is_end;
    switch(chatOperator->target){
        case TartgetType::RECVMSG:
            if(errorCode == 2){
                chatOperator->notifyUI("未检测到网络链接。请确认网络设置后再试。", chatOperator->is_end, chatOperator->result_index, errorCode);
                break;
            }
            chatOperator->notifyUI(chatOperator->result_msg, chatOperator->is_end, chatOperator->result_index, errorCode);
            if (chatOperator->soundFlag){
                //取消语音对话时自动朗读
//                emit chatOperator->sigSoundMsg(chatOperator->result_msg, chatOperator->is_end, chatOperator->result_index);
                if (chatOperator->is_end)
                    chatOperator->soundFlag = false;
            }

            break;
        case TartgetType::SUMMARY:
            emit chatOperator->sigGenerateSummary(chatOperator->result_msg, chatOperator->is_end, chatOperator->result_index, errorCode);
            break;
        case TartgetType::CALENDAR:
            emit chatOperator->sigAddToSchedule(chatOperator->result_msg, chatOperator->is_end, chatOperator->result_index, errorCode);
            break;
    }
}

ChatOperator::ChatOperator(QObject *parent)
    : QObject{parent},
      d(std::make_shared<ChatOperator::Private>())
{
}

ChatOperator::~ChatOperator()
{
    if (d->session)
        nlp_destory_session(d->session);
}

bool ChatOperator::reinitSession()
{
    closeSession();
    return true;
}

bool ChatOperator::sendMessage(const QString &message)
{
    if(d->session == nullptr){
        initSession();
    }
    QString send_message = message;
    QThread::create([this, send_message](){
        NlpResult ret = nlp_text_chat_async(d->session, send_message.toStdString().c_str());
        switch(ret){
            case NLP_SUCCESS:
                break;
            case NLP_NET_ERROR:
                KyInfo() << "NLP 网络设置错误 :" << ret;
                emit reply("未检测到网络链接。请确认网络设置后再试。", true, 0, ret);
                break;
            case NLP_SESSION_ERROR:
                KyInfo() << "session错误 :" << ret;
                emit reply("Session错误，请重启服务再试。", true, 0, ret);
                break;
            case NLP_SERVER_ERROR:
                KyInfo() << "服务错误或者不可用 :" << ret;
                emit reply("服务错误或者不可用。请切换网络再试。", true, 0, ret);
                break;
            case NLP_UNAUTHORIZED:
                KyInfo() << "权限限制，鉴权失败，无权访问 :" << ret;
                emit reply("鉴权失败，请重新配置KEY。", true, 0, ret);
                break;
            case NLP_REQUEST_TOO_MANY:
                KyInfo() << "请求频率过高 :" << ret;
                emit reply("请求频率过高，请等一会再试。", true, 0, ret);
                break;
            case NLP_REQUEST_FAILED:
                KyInfo() << "请求失败 :" << ret;
                emit reply("请求频率过高，请等一会再试。", true, 0, ret);
                break;
            case NLP_INPUT_TEXT_LENGTH_INVAILD:
                KyInfo() << "输入文本长度超限 :" << ret;
                emit reply("输入文本长度超限，请重新编辑文本再发送。", true, 0, ret);
                break;
            case NLP_PARAM_ERROR:
                KyInfo() << "参数错误 :" << ret;
                emit reply("鉴权失败，请重新配置KEY。", true, 0, ret);
                break;
            default:
                break;
        }
    })->start();

    return true;
}

void ChatOperator::notifyUI(const QString &message, bool is_end, int index, int errorCode)
{
    if (is_end) {
        emit sigSessionCompleted();
    }

    emit reply(message, is_end, index, errorCode);
}

void ChatOperator::setActor(int actorId)
{
    nlp_set_session_actor(d->session, actorId);
}

bool ChatOperator::initModelVendor()
{
    QJsonObject obj;
    obj.insert("apiKey", "0bLvOdqlo1Gzu2jbpf9imTsT");
    obj.insert("secretKey", "H0ThDOxNvTKEbNgyeHyKdw35R6dw3AcE");

    return set_remote_model_config(BAIDU, NLP, QJsonDocument(obj).toJson().toStdString().c_str());
}

bool ChatOperator::initSession()
{

    void *session = nullptr;
    // 创建session
    NlpResult ret = nlp_create_session(&session);
    if(session == nullptr){
        KyInfo() << "nlp create session error :" << ret;
        return false;
    }
    switch(ret){
        case NLP_NET_ERROR:
            KyInfo() << "NLP network settings error :" << ret;
            break;
        case NLP_SESSION_ERROR:
            KyInfo() << "NLP session error :" << ret;
            break;
        case NLP_SERVER_ERROR:
            KyInfo() << "NLP 服务错误或者不可用 :" << ret;
            break;
        case NLP_UNAUTHORIZED:
            KyInfo() << "NLP 权限限制，鉴权失败，无权访问 :" << ret;
            break;
        case NLP_REQUEST_TOO_MANY:
            KyInfo() << "NLP 请求频率过高 :" << ret;
            break;
        case NLP_REQUEST_FAILED:
            KyInfo() << "NLP 请求失败 :" << ret;
            break;
        case NLP_INPUT_TEXT_LENGTH_INVAILD:
            KyInfo() << "NLP 输入文本长度超限 :" << ret;
            break;
        case NLP_PARAM_ERROR:
            KyInfo() << "NLP 参数错误 :" << ret;
            break;
        default:
            break;
    }
    // 初始化session
    ret = nlp_init_session(session);
    switch(ret){
        case NLP_NET_ERROR:
            KyInfo() << "NLP network settings error :" << ret;
            break;
        case NLP_SESSION_ERROR:
            KyInfo() << "NLP session error :" << ret;
            break;
        case NLP_SERVER_ERROR:
            KyInfo() << "NLP 服务错误或者不可用 :" << ret;
            break;
        case NLP_UNAUTHORIZED:
            KyInfo() << "NLP 权限限制，鉴权失败，无权访问 :" << ret;
            break;
        case NLP_REQUEST_TOO_MANY:
            KyInfo() << "NLP 请求频率过高 :" << ret;
            break;
        case NLP_REQUEST_FAILED:
            KyInfo() << "NLP 请求失败 :" << ret;
            break;
        case NLP_INPUT_TEXT_LENGTH_INVAILD:
            KyInfo() << "NLP 输入文本长度超限 :" << ret;
            break;
        case NLP_PARAM_ERROR:
            KyInfo() << "NLP 参数错误 :" << ret;
            break;
        default:
            break;
    }
    if(!ret){
        d->session = session;
        nlp_set_result_callback(d->session, chatCallback, this);
    }

    return true;
}

void ChatOperator::closeSession()
{
    if (d->session)
        nlp_destory_session(d->session);

    d->session = nullptr;
}

void *ChatOperator::getSession()
{
    return d->session;
}

void ChatOperator::clearSession()
{
    if (d->session) {
        nlp_clear_context(d->session);
    }

    return;
}

void ChatOperator::stopChat()
{
    if (d->session) {
        //测试stop功能
        if (nlp_stop_chat(d->session) != NLP_SUCCESS)
            KyInfo() << "nlp stop chat error";
    }
    return;
}
